/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 20, 2001. A major rewrite was done in December 2004 to 
   - 
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2001-2007
   - HJ van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  HJ van Rantwijk <bugs4hj@netscape.net> (Original Author)
   -  Rajnish Bhaskar <raj@lordofthemoon.com>
   -
*/

/***
  * mzManagers is a JavaScript "class" with helper functions to open the Managers
  */
var mzManagers = 
{
  _nsIWindowMediator : Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator),

  mzGetPlatformExtras: function() 
  {
    var platformExtras = new Object();
    platformExtras.isClassicStyle = mzPreferences.readMyPref("multizilla.windows.classic-style", "bool", false);
    platformExtras.isMac = /Mac/.test(navigator.platform) || mzPreferences.readMyPref("multizilla.windows.mac-style", "bool", false);
    return platformExtras;
  },

  openPreferenceManager: function(aPanelID)
  {
    const _multiZilla = "MainPrefPanel";
    const _prefPath = "chrome://multiviews/content/preferences/";

    aPanelID = (aPanelID == undefined) ? _multiZilla : aPanelID;

    if (mzPreferences.readMyPref("multizilla.prefs.use-preference-manager", "bool", false) == false)
      goPreferences(_multiZilla, _prefPath + "multizillaMainPanel.xul", aPanelID);
    else {
      // Is the Preference Manager already opened?
      var prefWindow = mzGetMostRecentWindow("multizilla:preferences", false);

      if (prefWindow)
        prefWindow.focus();
      else {
        var winFeatures = "chrome, dialog, resizable"
        // The Preference Manager should be modal on Windows!
        if (/Win/.test(navigator.platform))
          winFeatures += ", dependent, modal";

        window.openDialog(_prefPath + "multizillaPreferenceWindow.xul", "", winFeatures, "", _multiZilla, aPanelID, this.mzGetPlatformExtras());
      }
    }
  },

  openSessionManager: function(aPrefFlag)
  {
    /***
      * Do a reverse lookup, otherwise we will always end up with 'multizilla:sessionmanager'
      * Note: keep the ID's of tsmIDs[] in sync with the XUL files for the windows!
      */
    var tsmIDs = [ "multizilla:tsm-renameSession", "multizilla:tsm-windowSelector", 
                   "multizilla:tabBehaviorSelection", "multizilla:sessionManager" ];

    for (i in tsmIDs) {
      var managerWin = mzGetMostRecentWindow(tsmIDs[i], false);

      if (managerWin) {
        if (i == 3) { // All windows that can open dialog windows
          var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
          var enumerator = windowManager.getEnumerator(null);

          while (enumerator.hasMoreElements()) {
            var win = enumerator.getNext();
            // Is addBookmark.xul open?
            if (win.document.documentElement.id == "newBookmarkDialog") {
              managerWin = win; // Yes, use that window
              break;
            }
            // Is there a confirmation dialog opened?
            if (win.document.documentElement.id == "commonDialog" && win.opener == managerWin) {
              managerWin = win; // Yes, use that window
              break;
            }
          }
        }
        managerWin.focus();
        return;
      }
    }
    window.openDialog("chrome://multiviews/content/session-manager/sessionManager.xul",  
                      "", "chrome, dialog, all", (aPrefFlag ? "preferences" : ""), this.mzGetPlatformExtras());
  },

  openUserAgentManager: function(aPrefFlag)
  {
    var managerWin = mzGetMostRecentWindow("multizilla:useragent-manager", false);

    if (managerWin) {
      var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
      var enumerator = windowManager.getEnumerator(null);

      while (enumerator.hasMoreElements()) {
        var win = enumerator.getNext();
        // Is there a confirmation dialog opened?
        if (win.document.documentElement.id == "commonDialog" && win.opener == managerWin) {
          managerWin = win; // Yes, use that window
          break;
        }
      }
      managerWin.focus();
      return;
    }
    window.openDialog("chrome://multiviews/content/useragent-manager/userAgentManager.xul", 
                      "", "chrome, dialog, all", (aPrefFlag ? "preferences" : ""), this.mzGetPlatformExtras());
  },

  openTabManager: function(aPrefFlag)
  {
    var managerWin = mzGetMostRecentWindow("multizilla:tabmanager", false);

    if (managerWin) {
      var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
      var enumerator = windowManager.getEnumerator(null);

      while (enumerator.hasMoreElements()) {
        var win = enumerator.getNext();
        // Is there a confirmation dialog opened?
        // if (win.document.documentElement.id == "commonDialog" && win.opener == dialogWin) {
        if (win.document.documentElement.id == "commonDialog" && 
		    managerWin.document.documentElement.id == win.opener.document.documentElement.id)
        {
          managerWin = win; // Yes, use that window
          break;
        }
      }
      managerWin.focus();
      return;
    }
    window.openDialog("chrome://multiviews/content/tab-manager/tabManager.xul", 
                      "", "chrome, alwaysRaised, dialog, all", (aPrefFlag ? "preferences" : ""), this.mzGetPlatformExtras());
  },

  openPermissionManager: function(aType, aHostPort, aPrefFlag)
  {
    /***
      * Do a reverse lookup, otherwise we will always end up with 'multizilla:permissionmanager'
      * Note: keep the ID's of pmIDs[] in sync with the XUL files for the windows!
      */
    var pmIDs = [ "multizilla:pm-filemanager", "multizilla:pm-prefwindow", 
                  "multizilla:permissionmanager" ];

    for (i in pmIDs) {
      var dialogWin = mzGetMostRecentWindow(pmIDs[i], false);
      // var permManagerWindow = this._nsIWindowMediator.getMostRecentWindow("multizilla:permissionmanager");

      if (dialogWin) {
        var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
        var enumerator = windowManager.getEnumerator(null);

        while (enumerator.hasMoreElements()) {
          var win = enumerator.getNext();
          // Is there a confirmation dialog opened?
          if (win.document.documentElement.id == "commonDialog" && win.opener == dialogWin) {
            dialogWin = win; // Yes, use that window
            break;
          }
        }
        try {
          dialogWin.focus();
        }
        catch(ex) {
          if (dialogWin.filePickerParent) {
            dump("\nID: " + dialogWin.filePickerParent.document.getElementsByTagName("radiogroup")[0].localName); // .documentElement.id);
            dialogWin.filePickerParent.document.getElementsByTagName("radiogroup")[0].focus();
          }
        }
      }
    }
    if (dialogWin) {
      return;
    }
    if (!aType) {
      var defaultType = mzPreferences.readMyPref("multizilla.perm-manager.default-manager", "string", "cookieManager");
      aType = defaultType;
    }
    if (aHostPort == undefined) {
      var browser = gBrowser.getBrowserForTab(gBrowser.selectedTab);

      if (browser) {
        var currentURI = browser.docShell.QueryInterface(Components.interfaces.nsIWebNavigation).currentURI;
        try {
          aHostPort = (currentURI.scheme == "file") ? "file:" : currentURI.hostPort;
        } catch(ex) {
          // Die silently
        }
      }
    }
    window.openDialog("chrome://multiviews/content/permission-manager/permissionManager.xul", 
                      "", "chrome, dialog, minimizable, resizable, centerscreen", aType, 
                      (aHostPort ? aHostPort : ""), (aPrefFlag ? "preferences" : ""), this.mzGetPlatformExtras());
  }
};

/***
  * mzQuickPrefsMenu is a JavaScript "class" with the helper functions used to  
  * initialize and handle all actions from the Quick Pref Menu's / menu items.
  */
var mzQuickPrefsMenu =
{
  _nsIPromptService : Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),
  _nsIPopupWindowManager : Components.classes["@mozilla.org/PopupWindowManager;1"].getService(Components.interfaces.nsIPopupWindowManager),
  _nsIPermissionManager : Components.classes["@mozilla.org/permissionmanager;1"].getService(Components.interfaces.nsIPermissionManager),
  _qpmBundle : srGetStrBundle("chrome://multiviews/locale/quickprefs/quickPrefsMenu.properties"),

  onQPMClick: function(aEvent, aPopupID)
  {
    if (aEvent.button == 0) {
      var i, menuitems;
      var popupNode = document.getElementById("quickpref-popup");

      if (aEvent.altKey) {
        this.openPreferenceEditor();
        return false;
      }
      else if (aEvent.metaKey || aEvent.ctrlKey) {
        var cachePopupNode = document.getElementById("quickpref-clearCachePopup");
        cachePopupNode.showPopup(cachePopupNode, aEvent.clientX, aEvent.clientY, "popup");
        return true;
      }
      else if (aEvent.shiftKey) {
        menuitems = popupNode.getElementsByAttribute("display", "menu");

        for (i = 0 ; i < menuitems.length; i++)
          menuitems[i].setAttribute("hidden", "true");

        popupNode.showPopup(popupNode, aEvent.clientX, aEvent.clientY, "popup");
        return false;
      }
      menuitems = popupNode.getElementsByAttribute("display", "menu");

      for (i = 0 ; i < menuitems.length; i++)
        menuitems[i].removeAttribute("hidden");

      popupNode.showPopup(popupNode, aEvent.clientX, aEvent.clientY, "popup");
      return true;
    }
    else if (aEvent.button == 1) 
      mzManagers.openPreferenceManager();
    else if (aEvent.button == 2) {
      var context = aEvent.target.getAttribute("context");

      if (context) {
        var contextNode = document.getElementById(context);
        contextNode.showPopup(contextNode, aEvent.clientX, aEvent.clientY, "popup");
        return true;
      }
      else {
        askToDisableButton(aEvent);
        return true;
      }
    }
    return false;
  },

  isMultiZillaRunning: function ()
  {
    var mzToolbar = document.getElementById("MultiZillaToolbar");

    if (mzToolbar)
      return mzToolbar;
    else 
      return null;
  },
  
  initQPMmenu: function(aPopupMenu)
  {
    var qpUAMenuPopup = document.getElementById("qpUAMenuPopup");

    if (!qpUAMenuPopup.ref) {
      qpUAMenuPopup.ref="urn:useragent-data:root";
      var uaStore = mzGetNewOrCurrentDSByFilename("useragents.rdf", false, null, false);

      if (uaStore)
        qpUAMenuPopup.database.AddDataSource(uaStore);
    }
    var i = 0;
    var mzToolbar = this.isMultiZillaRunning();
    var menuItems = aPopupMenu.getElementsByAttribute("display", "hidden-tabbar");
    // dump("\nmenuItems.length: " + menuItems.length);
    // Hide the <menuseparator> and the three extra <menuitem>'s
    for (i = 0; i < menuItems.length; i++) {
      menuItems[i].hidden = true;
    }
    if (mzToolbar) {
      var extraItems = 0;
      var mzToolbarHidden = (mzToolbar.getAttribute("hidden") == "true");
      var googleBoxOnly = (mzToolbar.mTabButtons.collapsed && mzToolbar.mToolButtons.collapsed && 
                           mzToolbar.mCloseButtons.collapsed);
      // Loop from bottom to top and display the extra menu items when the MultiZilla 
      // toolbar is hidden, or when the toolbar button is hidden. The extra menuseparator 
      // will only be diplayed when one of the extra menu items is displayed.
      for (i = (menuItems.length-1); i > -1; i--) {
        if (mzToolbarHidden || googleBoxOnly || mzToolbar.mTabButtons.childNodes[2].hidden || 
            (menuItems[i].id == "qpClearLocalDataMenu" && document.getElementById("privacymanager-tbbutton").hidden)) {
          menuItems[i].hidden = false;
        }
        if (mzToolbarHidden || googleBoxOnly ||mzToolbar.mTabButtons.childNodes[0].hidden || 
            (menuItems[i].id == "qpTabManager" && document.getElementById("tabmanager-tbbutton").hidden)) {
          menuItems[i].hidden = false;
          extraItems++;
        }
        else if (mzToolbarHidden || googleBoxOnly ||mzToolbar.mTabButtons.childNodes[1].hidden ||
                 (menuItems[i].id == "qpSessionManager" && document.getElementById("sessionmanager-tbbutton").hidden)) {
          menuItems[i].hidden = false;
          extraItems++;
        }
        else if (mzToolbarHidden || googleBoxOnly ||mzToolbar.mTabButtons.childNodes[3].hidden || 
                 (menuItems[i].id == "qpPermissionManager" && document.getElementById("permmanager-tbbutton").hidden)) {
          menuItems[i].hidden = false;
          extraItems++;
        }
        else if (extraItems && menuItems[i].localName == "menuseparator") {
          menuItems[i].hidden = false;
        }
      }
    } 
  },

  initGroupPref: function(aMenuElement)
  {
    var prefGroup = aMenuElement.getAttribute("prefgroup");
    var prefElements = document.getElementsByAttribute("prefgroup", prefGroup);

    if (prefElements.length == 0)
      return;

    var prefType = prefElements[1].getAttribute("preftype");
    var prefDefval = prefElements[1].getAttribute("prefdefval");
    var prefIdentifier = prefElements[1].getAttribute("prefstring");
    var prefValue = mzPreferences.readMyPref(prefIdentifier, prefType, 0, true);

    for (var i = 1; i < prefElements.length; i++) {
      if (prefElements[i].getAttribute("prefvalue") == prefValue) {
        prefElements[i].setAttribute("checked", "true");
        // prefElements[i].setAttribute("disabled", "true");
      }
      else {
        prefElements[i].removeAttribute("checked");
        // prefElements[i].removeAttribute("disabled");
      }
    }
  },

  initQuickPrefs: function()
  {
    var prefElements = document.getElementsByAttribute("pref", "true");

    for (var i = 0; i < prefElements.length; i++) {
      var prefId = prefElements[i].id;
      var prefType = prefElements[i].getAttribute("preftype");
      var prefIdentifier = prefElements[i].getAttribute("prefstring");
      var prefDefval = prefElements[i].getAttribute("prefdefval");
      var prefAttribute = prefElements[i].getAttribute("prefattribute");

      var prefValue = mzPreferences.readMyPref(prefIdentifier, prefType, prefDefval);

      if (prefAttribute == "checked") {
        if (prefValue >= 1)
          prefElements[i].setAttribute(prefAttribute, "true");
        else 
          prefElements[i].removeAttribute(prefAttribute);
      }
      else 
        prefElements[i].setAttribute(prefAttribute, prefValue);
    }
  },

  initSelectedQPrefsMenu: function(aElement)
  {
    var permission, url;
    var IDs = new Array();
    var baseIDs = [ "qpmBlock-", "qpmAllow-", "qpmDefault-" ];
    var menuPopupID = aElement.getAttribute("prefgroup");

    if (menuPopupID != "install")
      baseIDs[3] = "qpmSeparatorFor-";

    for (i in baseIDs) {
      IDs[i] = baseIDs[i] + menuPopupID;
    }

    if (menuPopupID == "cookies") {
      var managePrivacySettings = document.getElementById("qpm_ManageCookiesPrivacySettings");

      if (mzPreferences.readMyPref("network.cookie.cookieBehavior", "int", 3) == 3)
        managePrivacySettings.removeAttribute("hidden");
      else 
        managePrivacySettings.setAttribute("hidden", "true");
    }
    var uri = getBrowser().currentURI;
 
    if (gBrowser.selectedTab.hasAttribute("busy"))
      url = gBrowser.mCurrentTab.getAttribute("firstPage");
    else 
      url = gBrowser.mCurrentBrowser.webNavigation.currentURI.spec;

    permission = this._nsIPermissionManager.testPermission(uri, menuPopupID.replace(/s$/, ''));
    // selected menu items should be disabled and display a radio dot character!  
    this.enableElement(IDs[0], url, (permission == nsIPermissionManager.DENY_ACTION),
                       permission != nsIPermissionManager.DENY_ACTION);
    this.enableElement(IDs[1], url, (permission == nsIPermissionManager.ALLOW_ACTION),
                       permission != nsIPermissionManager.ALLOW_ACTION)
    if (IDs.length >= 2 && IDs[2])
      this.enableElement(IDs[2], url, (permission == nsIPermissionManager.UNKNOWN_ACTION), 
                         permission != nsIPermissionManager.UNKNOWN_ACTION);
    if (IDs.length >= 3 && IDs[3])
      this.enableElement(IDs[3], url, false, true);
  },

  enableElement: function(aElementID, aURL, aIsCurrentFlag, aIsEnabledFlag)
  {
    var element = document.getElementById(aElementID);

    if (element) {
      // Do we need to display a radio dot for this element?
      if (aIsCurrentFlag)
        element.setAttribute("currentPermission", "true"); // Yes we do!
      else
        element.removeAttribute("currentPermission"); // No, remove attribute

      // if (aIsEnabledFlag && aURL != "about:blank")
      if (aURL == "about:blank")
        element.setAttribute("disabled", "true");
      else
        element.removeAttribute("disabled");
    }
    else {
      throw("Unknown ID (" + aElementID + ") used in function initSelectedQPrefsMenu()");
    }
  },

  permissionUpdate: function(aElement, aURL)
  {
    var action = aElement.getAttribute("for");
    var permission = aElement.getAttribute("permission");

    if (permission == "none")
      permission = nsIPermissionManager.UNKNOWN_ACTION;
    else 
      permission = (aElement.getAttribute("permission") == "true") ? nsIPermissionManager.ALLOW_ACTION : nsIPermissionManager.DENY_ACTION;

    var url = (aURL == undefined) ? gBrowser.currentURI.spec : aURL;
    var isFileScheme = url.match(/(scheme:file|file:)/i);
    url = url.replace(/^\s*([-\w]*:\/+)?/, ""); // trim any leading space and scheme
    var host = isFileScheme ? 'file:' : 'http://' + url;
    const nsIIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var uri = nsIIOService.newURI(host, null, null);
    this._nsIPermissionManager.add(uri, action, permission);
    var message = aElement.getAttribute("msg");

    if (message)
      setTimeout(this.doAlert, 0, window, "", message);
  },

  changePref: function(aMenuElement, aMenuGroup, aRefreshFlag)
  {
    var prefType = aMenuElement.getAttribute("preftype");
    var prefIdentifier = aMenuElement.getAttribute("prefstring");
    var prefDefval = aMenuElement.getAttribute("prefdefval");
    var prefAttribute = null;
    var prefArray = new Array();
    var noAccess = "noAccess";
    var allAccess = "allAccess";
	
    if (prefIdentifier != null) {
      // retrieve current value
      var prefValue = mzPreferences.readMyPref(prefIdentifier, prefType, prefDefval);

      if (prefType == "bool") // flip flop the pref
        mzPreferences.writePref(prefType, prefIdentifier, !prefValue);
      else {
        if (aMenuGroup) // special case, use this value
          prefValue = aMenuElement.getAttribute("prefvalue");
        else { // used for int and string, other might follow
          prefAttribute = aMenuElement.getAttribute("prefattribute");

          if (prefAttribute == "checked") {
            // setup an array with values to use
            var prefValues = eval(aMenuElement.getAttribute("prefvalue"));
            prefArray = prefValues;
            // pick the first or second value, based on the "checked" state
            prefValue = aMenuElement.getAttribute(prefAttribute) ? prefArray[1] : prefArray[0];
          }
          else
            prefValue = aMenuElement.getAttribute(prefAttribute);
        }
        // Ok, lets write this sucker
        mzPreferences.writePref(prefType, prefIdentifier, prefValue);
      }
      if (aRefreshFlag) // this pref setting needs a refresh
        getWebNavigation().reload(nsIWebNavigation.LOAD_FLAGS_IS_REFRESH);
    }
  },

  initUAMenuPopup: function(aMenuPopup)
  {
    aMenuPopup.builder.rebuild();

    var customUA = false;
    var restoreOriginalUA = document.getElementById("qpm-restoreOriginalUA");
    var userAgents = aMenuPopup.childNodes.length; 

    for (var i = 4; i < userAgents; i++) {
      var menuitem = aMenuPopup.childNodes[i];
      var menuitemUserAgent = menuitem.getAttribute("useragent");

      if (menuitem.label == "")
        menuitem.label = menuitemUserAgent;
      if (!customUA && (menuitemUserAgent == navigator.userAgent)) {
        menuitem.setAttribute("checked", "true");
        restoreOriginalUA.removeAttribute("disabled");
        restoreOriginalUA.removeAttribute("checked");
        customUA = true;
      }
    }
    if (!customUA) {
      restoreOriginalUA.setAttribute("disabled", "true");
      restoreOriginalUA.setAttribute("checked", "true");
    }
  },

  setUserAgentOverride: function(aEvent)
  {
    const PREFSTRING = "multizilla.general.useragent.override";
    var menuitem = aEvent.originalTarget;
    var menuitemID = menuitem.getAttribute("id");

    if (menuitemID == "qpm-restoreOriginalUA")
      mzPreferences.writePref("char", PREFSTRING, "");
    else {
      var useragent = menuitem.getAttribute("useragent");

      if (useragent)
        mzPreferences.writePref("char", PREFSTRING, useragent);
    }
  },

  initRefSpoofingMenu: function(aElement)
  {
    var refSetting = -1;
    var spoofURL = "http://ref.spoof.org/refspoof.cgi"; // default fake referrer
    var aWindow = gBrowser.contentWindow;
    var aDocument = aWindow.document;

    try { 
      refSetting = pref.getIntPref("network.http.sendRefererHeader");
    } catch (ex) {
    }
    for (var i = 0; i < aElement.childNodes.length; i++) {
      aElement.childNodes[i].setAttribute("disabled", (refSetting == 0) ? "true" : "false");
      aElement.childNodes[i].setAttribute("checked", "false");
    }
    try {
      spoofURL = mzPreferences.readMyPref("multizilla.refspoof.spoofurl", "string", spoofURL);
    } catch (ex) {
    }
    aElement.childNodes[6].setAttribute("label", this._qpmBundle.GetStringFromName("fake.label") + " " + spoofURL);
    aElement.childNodes[7].setAttribute("label", aDocument.referrer ?  
                                                 this._qpmBundle.GetStringFromName("currentRef.label") + " " + aDocument.referrer : 
                                                 this._qpmBundle.GetStringFromName("noRef.label"));

    if (refSetting)
      this.initGroupPref(aElement);
  },

  displayMessageByNumber: function(aMessageNumber)
  {
    function delayedStartup() {
      var message = null;

      switch(aMessageNumber)
      {
        case 0: message = mzQuickPrefsMenu._qpmBundle.GetStringFromName("refSpooferDisabled");
          break;
        case 1: message = mzQuickPrefsMenu._qpmBundle.GetStringFromName("refSpooferEnabled");
          break;
        case 2: message = mzQuickPrefsMenu._qpmBundle.GetStringFromName("refSpooferNone");
          break;
        case 3: message = mzQuickPrefsMenu._qpmBundle.GetStringFromName("illegalFakeReferrer");
          break;
        case 4:
        case 5: var spoofer = mzPreferences.readMyPref("multizilla.refspoof.spoofing", "int", 1);
                message = (spoofer == 1) ? "refSpooferDisabled" : "refSpooferEnabled";
                mzQuickPrefsMenu._qpmBundle.GetStringFromName(message);
          break;
      }
      if (message) {
        var dialogTitle = mzQuickPrefsMenu._qpmBundle.GetStringFromName("fakeReferrerDialogTitle");
        mzQuickPrefsMenu.doAlert(window, dialogTitle, message);
      }
    }
    setTimeout(delayedStartup, 0);
  },

  manageFakeReferrer: function()
  {
    var bundle = mzQuickPrefsMenu._qpmBundle;
    var dialogTitle = bundle.GetStringFromName("fakeReferrerDialogTitle");
    var dialogText = bundle.GetStringFromName("fakeReferrerDialogText");
    var fakeRefIsText = bundle.GetStringFromName("fakeRefIsSetTo");
    var fakeReferrer = mzPreferences.readMyPref("multizilla.refspoof.spoofurl", "url", "http://ref.spoof.org/refspoof.cgi");
    var result = {value:fakeReferrer};

    var ret = mzQuickPrefsMenu._nsIPromptService.prompt(window, dialogTitle, dialogText, result, null, {value:0});

    if (ret && result.value && result.value != "") {
      mzQuickPrefsMenu.doAlert(window, dialogTitle, fakeRefIsText+result.value);
      mzPreferences.writePref("string", "multizilla.refspoof.spoofurl", result.value);
    }
    else
      mzQuickPrefsMenu.displayMessageByNumber(3);
  },

  initExtensionsMenu: function(aMenuPopup)
  {
    var enabled = 0;
    var disabled = 0;
    var installed = 0;
    // menuitem, menuitem, menuseparator, menuitem, template
    for (var i = (aMenuPopup.childNodes.length - 1); i > 4; i--) {
      if (aMenuPopup.childNodes[i].hasAttribute("injected"))
        aMenuPopup.removeChild(aMenuPopup.childNodes[i]);
    }
    aMenuPopup.builder.rebuild();

    for (var i = 4; i < aMenuPopup.childNodes.length; i++) {
      installed++;
      var packageName = aMenuPopup.childNodes[i].getAttribute("packageName");
      var packageState = (aMenuPopup.childNodes[i].getAttribute("checked") == "true");

      if (!packageState)
        disabled++;
      else if (packageName != "multiviews")
        enabled++;
    }
    var disableState = (installed == 1 || enabled == 0 || disabled == installed) ? "true" : "false";
    aMenuPopup.childNodes[0].setAttribute("disabled", disableState); // 'Disable all Extensions'
    disableState = (installed == 1 || disabled == 0 || enabled == installed) ? "true" : "false";
    aMenuPopup.childNodes[1].setAttribute("disabled", disableState); // 'Enable all Extensions'

    var menuseparator = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuseparator");
    menuseparator.setAttribute("injected", "true");
    aMenuPopup.appendChild(menuseparator);

    var bundle = mzQuickPrefsMenu._qpmBundle;
    var menuitem = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
    menuitem.setAttribute("injected", "true");
    menuitem.setAttribute("label", bundle.GetStringFromName("qpPermissionManager.label"));
    menuitem.setAttribute("accesskey", bundle.GetStringFromName("qpPermissionManager.accesskey"));
    menuitem.setAttribute("acceltext", bundle.GetStringFromName("qpPermissionManager.acceltext"));
    menuitem.setAttribute("oncommand", "event.stopPropagation();mzManagers.openPermissionManager('installManager-Preferences');");
    aMenuPopup.appendChild(menuitem);
  },

  /***
    * Helper function for: toggleExtensionEnabledState() and setAllExtensionsTo()
    */
  setExtensionStateTo: function(packageName, packageState)
  {
    var nsIChromeRegistry = null;

    if ('nsIChromeRegistrySea' in Components.interfaces)
      nsIChromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIChromeRegistrySea);
    else if ('nsIXULChromeRegistry' in Components.interfaces)
      nsIChromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIXULChromeRegistry);

    packageName = packageName.toLowerCase()
    nsIChromeRegistry.setAllowOverlaysForPackage(packageName, packageState);

    if (packageName == "multiviews") {
      var prefStrings = [ "xpinstall.dialog.confirm", "general.useragent.vendor", "general.useragent.vendorSub" ];
      // Are we about to reactivate MultiZilla?
      if (packageState) {
        mzPreferences.writePref("string", prefStrings[0], "chrome://multiviews/content/xpinstall/xpinstallConfirm.xul");
        mzPrefInit.checkVendorPrefs();
      }
      else { // Deactivate MultiZilla
        // Clear MultiZilla's user settings!
        for (i in prefStrings) {
          if (pref.prefHasUserValue(prefStrings[i]))
            pref.clearUserPref(prefStrings[i]);
        }
      }
    }
    else if (packageName == "googlebox") {
      mzPreferences.writePref("bool", "multizilla.integrated.toolbar", false);
    }
  },

  /***
    * Helper function for: toggleExtensionEnabledState() and setAllExtensionsTo()
    */
  openExtensionManagment: function(packages, aNewState)
  {
    var dialogWin = mzGetMostRecentWindow("multizilla:extension-management", false);

    if (dialogWin) {
      dialogWin.focus();
      return;
    }
    var winFeatures = "chrome, dialog, centerscreen, resizable, dependent, modal";
    window.openDialog("chrome://multiviews/content/quickprefs/miscellaneous/extensionManagement.xul",
                      "", winFeatures, packages, aNewState, mzManagers.mzGetPlatformExtras());
  },

  toggleExtensionEnabledState: function(aEvent)
  {
    var element = aEvent.originalTarget;
    var packageName = element.getAttribute("packageName");
    var packageState = Boolean(element.getAttribute("checked"));

    mzQuickPrefsMenu.openExtensionManagment([packageName], !packageState);
  },

  setAllExtensionsTo: function(aMenupopup, aNewState)
  {
    var packages = new Array();
    /***
      * Grap all target packages from aMenupopup
      * First package in aMenupopup will be the fourth item
      */
    for (var i = 4; i < aMenupopup.childNodes.length; i++) {
      var packageName = aMenupopup.childNodes[i].getAttribute("packageName");

      if (aNewState == true && packageName == "multiviews") {
        continue;
      }
      else {
        var packageState = (aMenupopup.childNodes[i].getAttribute("checked") == "false");

        if (packageState != aNewState) {
          packageName = packageName.toLowerCase();
          packages.push(packageName);
        }
      }
    }
    if (packages.length > 0) {
      mzQuickPrefsMenu.openExtensionManagment(packages, aNewState);
    }
  },

  relaunchBrowserWindows: function(aCleanRestartFlag)
  {
    var windowManagerDS = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService(nsIWindowDataSource);
    var currentBrowserWindows = mzGetRealWindowResourceIDList();
    var targetWindow = null;

    if (aCleanRestartFlag)
    {
      window.open();

      for (i in currentBrowserWindows) {
        targetWindow = windowManagerDS.getWindowForResource(currentBrowserWindows[i]);
        targetWindow.close();
      }
    }
    else
    {
      var sourceWindow = null;

      for (i in currentBrowserWindows) {
        var targetTabbrowser = openWindowAndReturnBrowser();
        targetWindow = targetTabbrowser.ownerDocument.defaultView;
        sourceWindow = windowManagerDS.getWindowForResource(currentBrowserWindows[i]);
        var sourceTabbrowser = sourceWindow.getBrowser();

        if (!sourceWindow.areAllTabsBlank()) {
          var tabs = sourceTabbrowser.mTabs;
          var selectedIndex, selectedTab = sourceTabbrowser.selectedTab;

          for (var t = (tabs.length-1); t >= 0; t--) {
            var openBrowserWindows = mzGetRealWindowResourceIDList();
            var windowID = openBrowserWindows[openBrowserWindows.length-1]

            if (tabs[t] == selectedTab)
              selectedIndex = t;

            sourceWindow.moveTabToWindow(tabs[t], windowID, true, true);
          }
          tabs = targetTabbrowser.mTabs;

          for (t = 0; t < tabs.length; t++) {
            if (tabs[t].getAttribute("firstPage") == 'about:blank') {
              targetTabbrowser.removeTab(tabs[t]);
            }
          }
          var removedTabs = sourceTabbrowser.mRemovedTabs;

          if (removedTabs.length > 0)
          {
            for (rIndex in removedTabs) {
              var removedTabData = removedTabs[rIndex];
              var removedTab = new Object();
              removedTab.firstPage          = removedTabData.firstPage;
              removedTab.customLabel        = removedTabData.customLabel;
              removedTab.reloadInterval     = removedTabData.reloadInterval;
              removedTab.allowPlugins       = removedTabData.allowPlugins;
              removedTab.allowJavascript    = removedTabData.allowJavascript;
              removedTab.allowMetaRedirects = removedTabData.allowMetaRedirects;
              removedTab.allowSubframes     = removedTabData.allowSubframes;
              removedTab.allowImages        = removedTabData.allowImages;
              removedTab.history            = removedTabData.history;
              targetTabbrowser.mRemovedTabs[rIndex] = removedTab;
            }
          }
          setTimeout(function selectTab(aTabbrowser, aIndex){aTabbrowser.selectedTab = aTabbrowser.mTabs[aIndex];}, 0, targetTabbrowser, selectedIndex);
        }
        sourceWindow.close();
      }
    }
  },

  openPreferenceEditor: function()
  {
    gBrowser.addOrGetTab("chrome://global/content/config.xul", undefined, undefined, true, true); 
    gBrowser.userTypedValue = ""; 
    SetPageProxyState("invalid", null);
  },

  doAlert: function(aWindow, aTitle, aText)
  {
    mzQuickPrefsMenu._nsIPromptService.alert(aWindow, aTitle, aText);
  },

  getConfirmation: function(aConfirmationTitle, aConfirmationText, aWindow) // mzConfirmAction
  {
    if (aWindow == undefined)
      aWindow = window;
    // dump("\naConfirmationTitle: " + aConfirmationTitle + "\naConfirmationText: " + aConfirmationText);
    var dialogTitle, dialogText;

    try {
      dialogTitle = this._qpmBundle.GetStringFromName(aConfirmationTitle);
    } catch(ex) {
      dialogTitle = aConfirmationTitle;
    }
    try {
      dialogText = this._qpmBundle.GetStringFromName(aConfirmationText);
    } catch(ex) {
      dialogText = aConfirmationText;
    }
    return this._nsIPromptService.confirm(aWindow, dialogTitle, dialogText);
  }
};

/***
  * mzPrivacyTools is a JavaScript "class" with the helper functions to 
  * remove the stored 'privacy sensitive data' from memory and/or harddrive.
  */
var mzPrivacyTools =
{
  _nsICacheService : Components.classes["@mozilla.org/network/cache-service;1"].getService(Components.interfaces.nsICacheService),
  _nsICookieManager : Components.classes["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager),
  _nsIPasswordManager : Components.classes["@mozilla.org/passwordmanager;1"].getService(Components.interfaces.nsIPasswordManager),
  _nsIDownloadManager : Components.classes["@mozilla.org/download-manager;1"].getService(Components.interfaces.nsIDownloadManager),
  _nsIWalletEditor : Components.classes["@mozilla.org/walleteditor/walleteditor-world;1"].createInstance(Components.interfaces.nsIWalletEditor),

  clearDiskMemCache: function()
  {
    this.clearCache("BOTH");
  },

  clearAllBackTraces: function()
  {
    this.clearGlobalHistory();
    this.clearUrlBarHistory();
    this.clearCache("BOTH");
    mzPurgeIDNHistory();
  },

  clearGlobalHistory: function()
  {
    var globalHistory;
    // Builds 20040210 and up (see also mozilla bug 224829) 
    if ("@mozilla.org/browser/global-history;2" in Components.classes)
      globalHistory = Components.classes["@mozilla.org/browser/global-history;2"];
    else 
      globalHistory = Components.classes["@mozilla.org/browser/global-history;1"];

    globalHistory = globalHistory.getService(Components.interfaces.nsIBrowserHistory);
    globalHistory.removeAllPages();
  },

  clearUrlBarHistory: function()
  {
    // New implementation, this only works in Mozilla builds 20040602 and later
    // see also: https://bugzilla.mozilla.org/show_bug.cgi?id=245009
    mzPreferences.writePref("char", "general.open_location.last_url", "")
    var localStore = RDF.GetDataSource("rdf:local-store");
    var urlBarHist = mzRDFCUtils.MakeSeq(localStore, RDF.GetResource("nc:urlbar-history"));

    for (var i = urlBarHist.GetCount(); i > 0; --i) {
      urlBarHist.RemoveElementAt(i, true);
    }
  },

  clearCache: function(aCacheType)
  {
    if (aCacheType == "DISK")
      this._nsICacheService.evictEntries(Components.interfaces.nsICache.STORE_ON_DISK);
    else if (aCacheType == "MEMORY")
      this._nsICacheService.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
    else { // "BOTH"
      this._nsICacheService.evictEntries(Components.interfaces.nsICache.STORE_ON_DISK);
      this._nsICacheService.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
    }
  },

  clearAllCookies: function()
  {
    this._nsICookieManager.removeAll();
  },
  ////////////////////////////////////////////////////////////////////////////////
  // This clears all saved passwords. This is a serious action (dataloss at risk) 
  // so we require the users confirmation, before doing anything. There's also a 
  // pref "wallet.crypto" that can be used to encrypt sensitive data (passwords)
  // But, we don't have to check this pref, because mozilla will take care of it!
  ////////////////////////////////////////////////////////////////////////////////
  // http://lxr.mozilla.org/seamonkey/source/netwerk/base/public/nsIPasswordManager.idl
  clearAllPasswords: function()
  {
    var sDecoderRing = Components.classes["@mozilla.org/security/sdr;1"].getService(Components.interfaces.nsISecretDecoderRing);
    sDecoderRing.encryptString("abc"); // trigger master password prompt!

    var container = this._nsIPasswordManager.enumerator;
    var items = [];

    while (container.hasMoreElements()) {
      var item = container.getNext().QueryInterface(Components.interfaces.nsIPassword);
      items.push(item);
    }
    for (var i = 0; i < items.length; ++i)
      this._nsIPasswordManager.removeUser(items[i].host, items[i].user);
  },

  clearAllDownloads: function()
  {
    // Construct the datasource from the filename, because I can't find it in the source!
    var datasource = mzGetNewOrCurrentDSByFilename("downloads.rdf", false, null, false);
    var downloads = mzConstructRDFContainer(datasource, "seq", "NC:DownloadsRoot", false).GetElements(); 
    var downloadQueue = new Array();

    while (downloads.hasMoreElements()) {
      var item = downloads.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
      downloadQueue.push(item);
    }
    this._nsIDownloadManager.startBatchUpdate();

    for (var i = 0; i < downloadQueue.length; i++) {
      try {
        this._nsIDownloadManager.removeDownload(downloadQueue[i].Value);
      } catch(ex) {
        // dump("\nmzClearAllDownloads() failed to remove download data!");
      }
    }
    this._nsIDownloadManager.endBatchUpdate();
  },

  clearAllFormData: function()
  {
    var list = this._nsIWalletEditor.GetValue();

    if (list.length > 0) { // no data available or database locked!
      var BREAK = list[0];
      this._nsIWalletEditor.SetValue("OK" + BREAK, window);
    }
  },

  clearAllLocalData: function()
  {
    /***
    * This can be used to clear all passwords and form data!
    * var walletServiceInterface = Components.classes['@mozilla.org/wallet/wallet-service;1'].getService(Components.interfaces.nsIWalletService);
    * walletServiceInterface.WALLET_DeleteAll(); 
    */
    this.clearCache("ALL");
    this.clearUrlBarHistory()
    this.clearGlobalHistory();
    this.clearAllCookies();
    this.clearAllDownloads();
    this.clearAllFormData();
    this.clearAllPasswords();
    mzPurgeIDNHistory();
  },

  clearAfterConfirmation: function(aElement)
  {
    switch(aElement.id || aElement.observes) {
      case "ClearDiskMemoryCache":
        if (mzQuickPrefsMenu.getConfirmation('qpClearMemDiskTitle', 'qpClearMemDiskText')) 
          mzPrivacyTools.clearDiskMemCache();
        break;
      case "ClearAllCache":
        if (mzQuickPrefsMenu.getConfirmation('qpClearAllTitle', 'qpClearAllText')) 
          mzPrivacyTools.clearAllBackTraces();
        break;
      case "ClearAllCookies":
        if (mzQuickPrefsMenu.getConfirmation('qpRemoveAllCookiesTitle', 'qpRemoveAllCookiesText')) 
          mzPrivacyTools.clearAllCookies();
        break;
      case "ClearAllForms":
        if (mzQuickPrefsMenu.getConfirmation('qpClearAllFormDataTitle', 'qpClearAllFormDataText')) 
          mzPrivacyTools.clearAllFormData();
        break;
      case "ClearAllPasswords":
        if (mzQuickPrefsMenu.getConfirmation('qpClearAllPasswordsTitle', 'qpClearAllPasswordsText'))
          mzPrivacyTools.clearAllPasswords();
        break;
      case "ClearIDNHistory":
        if (mzQuickPrefsMenu.getConfirmation('qpClearIDNHistoryTitle', 'qpClearIDNHistoryText')) 
          mzPurgeIDNHistory();
        break;
      case "ClearAllLocalData":
        if (mzQuickPrefsMenu.getConfirmation('qpClearAllDataTitle', 'qpClearAllDataText'))
          mzPrivacyTools.clearAllLocalData();
        break;
    }
  }
};

